<?php 
    /**
     * Load the Js required to init the ratings and review code from Amazon. 
     * This needs to load last after all widget code. 
     */
    add_action( 'woocommerce_after_single_product', 'abwp_load_amazon_js', 6 );
    function abwp_load_amazon_js() {
        // Make sure this plugin is enabled before loading any JS code.
        if (abwp_plugin_status() != 0) {
            echo "
                <!-- Beginning of Buy With Prime Widget -->
                <script async fetchpriority='high' src='https://code.buywithprime.amazon.com/bwp.js'></script>
                <!-- End of Buy With Prime Widget -->
            ";
        }
    }

    /**
     * Check if this plugin is disabled  
     */
    function abwp_plugin_status() {
        $status = 0;
        if (!empty(get_option( 'abwp_ratings_html' )) && get_option( 'abwp_rating_location' ) != 'disabled' ) {
            $status = 1;
        }
        if (!empty(get_option( 'abwp_reviews_html' )) && get_option( 'abwp_review_location' ) != 'disabled' ) {
            $status = 1;
        }
        if (!empty(get_option( 'abwp_button_html' )) && get_option( 'abwp_button_location' ) != 'disabled' ) {
            $status = 1;
        }
        return $status;
    }

    /**
     * Inject the Amazon Product Rating in the woocommerce template, based on admin settings.
     */
    add_action( 'template_redirect', 'abwp_load_amazon_rating' );
    function abwp_load_amazon_rating() {
        if (!empty(get_option( 'abwp_ratings_html' )) && get_option( 'abwp_rating_location' ) != 'disabled' ) {
            $location = get_option( 'abwp_rating_location' );
            if ($location == 'before_product_title') {
                add_action( 'woocommerce_single_product_summary', 'abwp_load_amazon_rating_html', 1 );
            }
            if ($location == 'after_product_title') {
                add_action( 'woocommerce_single_product_summary', 'abwp_load_amazon_rating_html', 6 );
            }
            if ($location == 'after_short_description') {
                add_action( 'woocommerce_before_add_to_cart_form', 'abwp_load_amazon_rating_html', 1 );
            }
            if ($location == 'before_add_to_cart') {
                add_action( 'woocommerce_before_add_to_cart_button', 'abwp_load_amazon_rating_html', 1 );
            }
            if ($location == 'after_add_to_cart') {
                add_action( 'woocommerce_after_add_to_cart_button', 'abwp_load_amazon_rating_html', 1 );
            }
        }
    }

    /**
     * Load JS scripts
     */
    function abwp_enqueue_custom_scripts() {
        if (is_product()) {
            wp_enqueue_script(
                'abwp-main-script', 
                plugins_url('/js/abwp-main.js', __FILE__), 
                array('jquery', 'woocommerce', 'wc-add-to-cart'),
                null, 
                true
            );
        }
    }
    add_action('wp_enqueue_scripts', 'abwp_enqueue_custom_scripts');    

    /**
     * Display the Amazon ratings for a product, supporting both simple and variable products.
     */
    function abwp_load_amazon_rating_html() { 
        global $product;

        // Initialize the SKU variable and a flag to detect if the product is a variation
        $product_sku = '';
        $is_variation = false;

        // Check if the product is a variable product
        if ($product->is_type('variable')) {
            // Get the available variations
            $available_variations = $product->get_available_variations();
            
            // Check if there is a default variation set, fetch its SKU
            if (!empty($available_variations)) {
                // Assuming the default variation is the first one
                $default_variation = reset($available_variations);
                $product_sku = $default_variation['sku'];
                $is_variation = true;  // Set flag as this is a variation
            }
        } else {
            // Fetch the SKU for a simple product
            $product_sku = $product->get_sku();
        }

        // Generate a dynamic admin message based on product type
        $product_type_message = $is_variation ? "variation does" : "product does";

        // Provide a clearer, more professional admin message when SKU is missing
        if (empty($product_sku) && current_user_can('administrator')) {
            echo "<strong>Admin Notice:</strong> This $product_type_message not have an SKU set. The 'Buy with Prime' button cannot be displayed.<br>";
            echo "<small>This warning is only visible to administrators.</small>";
            return;
        } 

        // Display the ratings HTML replacing the placeholder with the actual SKU
        echo abwp_replace_product_sku(get_option('abwp_ratings_html'), $product_sku);
    }


    /**
     * Replace the product sku in the amazon buy with prime code. 
     */
    function abwp_replace_product_sku( $string, $product_sku ) {
        $string = str_replace( '{{product.sku}}', $product_sku, $string );
        return $string;
    }

    /**
     * Inject the Amazon Product reviews in the woocommerce template, based on admin settings.
     */
    add_action( 'template_redirect', 'abwp_load_amazon_reviews' );
    function abwp_load_amazon_reviews() {
        if (!empty(get_option( 'abwp_reviews_html' )) && get_option( 'abwp_review_location' ) != 'disabled' ) {
            $location = get_option( 'abwp_review_location' );
            if ($location == 'replace') {
                // Replace the tab title because the review count from wordpress will not match amazon. 
                add_filter( 'woocommerce_product_tabs', 'adwp_rename_reviews_tab' );
                add_filter( 'woocommerce_product_tabs', 'abwp_replace_reviews_tab' );
            }
            //add_action( $location, 'abwp_load_amazon_review_html', 6 );
        }
    }

    /**
     * Inject the Amazon Buy With Prime Button.
     */
    add_action( 'template_redirect', 'abwp_load_amazon_button' );
    function abwp_load_amazon_button() {

        if (!empty(get_option( 'abwp_button_html' )) && get_option( 'abwp_button_location' ) != 'disabled' ) {
            $location = get_option( 'abwp_button_location' );
            if ($location == 'after_add_to_cart') {
                add_action( 'woocommerce_after_add_to_cart_button', 'abwp_load_amazon_button_html', 1 );
            }
            if ($location == 'before_add_to_cart') {
                add_action( 'woocommerce_before_add_to_cart', 'abwp_load_amazon_button_html', 1 );
            }
            if ($location == 'after_short_description') {
                add_action( 'woocommerce_after_short_description', 'abwp_load_amazon_button_html', 1 );
            }
            if ($location == 'after_add_to_cart_form') {
                add_action( 'woocommerce_after_add_to_cart_form', 'abwp_load_amazon_button_html', 1 );
            }
            //add_action( $location, 'abwp_load_amazon_review_html', 6 );
        }
    }



    /**
     * Display the Amazon Button
     */
/**
 * Display the Amazon Buy with Prime button, supporting both simple and variable products.
 * Includes checks on all variations for SKUs and alerts administrators if any variations are missing an SKU.
 */
function abwp_load_amazon_button_html() { 
    global $product;

    $product_sku = $product->get_sku(); // Default to the main product SKU
    $is_variation = false;
    $missing_sku_warning = false;

    // Handle variable products
    if ($product->is_type('variable')) {
        $available_variations = $product->get_available_variations();
        $is_variation = true;
        $variation_skus = array_filter(array_map(function ($variation) {
            return $variation['sku'];
        }, $available_variations));

        // Check if any variations are missing an SKU
        if (count($variation_skus) < count($available_variations)) {
            $missing_sku_warning = true;
        }

        // Use the first variation's SKU as the default if available
        if (!empty($variation_skus)) {
            $product_sku = reset($variation_skus);
        }
    }

    // Admin-specific notices
    if (current_user_can('administrator')) {
        if ($missing_sku_warning) {
            echo "<strong>Admin Notice:</strong> One or more variations do not have an SKU set. This may impact Buy with Prime functionality.<br>";
        }
        if (empty($product_sku)) {
            echo "<strong>Admin Notice:</strong> This product does not have a primary SKU set and cannot display the 'Buy with Prime' button.<br>";
            echo "<small>This warning is only visible to administrators.</small>";
            return;
        }
    } 

    // Output the Buy with Prime button with the SKU dynamically replaced, if available
    if (!empty($product_sku)) {
        echo abwp_replace_product_sku(get_option('abwp_button_html'), $product_sku);
    } else {
        echo "<p>This product is not currently eligible for Buy with Prime as it lacks a valid SKU.</p>";
    }
}



    /**
     * Replace the default woocomerce reviews with Amazon Reviews. 
     */
    function abwp_replace_reviews_tab( $tabs ) {
        $tabs[ 'reviews' ][ 'callback' ] = 'abwp_load_amazon_review_html';
        return $tabs;
    }
    /**
     * Replace the default woocomerce reviews title.
     */
    function adwp_rename_reviews_tab( $tabs ) {
        $tabs[ 'reviews' ][ 'title' ] = 'Reviews';
        return $tabs;
    }

    /**
     * Display The Amazon Reviews
     */
    function abwp_load_amazon_review_html() { 
        global $product; 
        $product_sku = $product->get_sku();
        if (empty($product_sku) && current_user_can('administrator') ) {
            echo "<b>IMPORTANT:</b> This product will not display the Amazon Buy with Prime button because it does not have an SKU set.<br>";
            echo "<small>This message is only visible to logged in administrators.</small>";
            return;
        } 
        echo abwp_replace_product_sku(get_option( 'abwp_reviews_html' ), $product_sku);
    }
